*
* $Id: box_intercept.F,v 1.2 1995/08/29 13:28:20 zfiles Exp $
*
* $Log: box_intercept.F,v $
* Revision 1.2  1995/08/29 13:28:20  zfiles
* Complete new set from Rob/Anders
*
*
C
C     Code created by 
C
C     Robert Kutschke : kutschke@lns598.lns.cornell.edu
C     Anders Ryd      : ryd@charm.physics.ucsb.edu
C
C     Created : Sept 1994 -- June 1995
C     Title   : box_intercept
C     Purpose : This routine finds where a straight line from x1 to x2
C               intercepts a box with half sides dx,dy, and dz. The
C               fraction of the distance gone from x1 towards x2 is
C               returned in s_list and in_out_list flags if the point
C               was an entrance (+1) or an exit (-1). n_entries is the 
C               number of exits+entrances. Cmp tub_intercept.
C
C     
C
C     Modifications:
C
C
C


      subroutine box_intercept(s_list,in_out_list,n_entries,
     +                             dx,dy,dz,x1,x2)

      implicit none

      integer in_out_list(4),n_entries

      real s_list(4),dx,dy,dz,x1(3),x2(3)
 
      logical in_1,in_2
   

      real s,v(3),xl,yl,zl

      
      in_1=((abs(x1(1)).lt.dx).and.(abs(x1(2)).lt.dy).and.
     +                             (abs(x1(3)).lt.dz))

      in_2=((abs(x2(1)).lt.dx).and.(abs(x2(2)).lt.dy).and.
     +                             (abs(x2(3)).lt.dz))



      if (in_1.and.in_2) then

        n_entries=0

      elseif((.not.in_1).and.(.not.in_2)) then

        v(1)=x2(1)-x1(1)
        v(2)=x2(2)-x1(2)
        v(3)=x2(3)-x1(3)

        s_list(1)=-1.0

        n_entries=0

        if ((x1(1).gt.dx.and.x2(1).lt.dx)) then
           s=(dx-x1(1))/v(1)
           yl=x1(2)+s*v(2)
           zl=x1(3)+s*v(3)
           if (abs(yl).lt.dy.and.abs(zl).lt.dz) s_list(1)=s
        endif

        if ((x1(1).lt.dx.and.x2(1).gt.dx)) then
           s=(dx-x1(1))/v(1)
           yl=x1(2)+s*v(2)
           zl=x1(3)+s*v(3)
           if (abs(yl).lt.dy.and.abs(zl).lt.dz) s_list(2)=s
        endif

        if ((x1(1).gt.-dx.and.x2(1).lt.-dx)) then
           s=(-dx-x1(1))/v(1)
           yl=x1(2)+s*v(2)
           zl=x1(3)+s*v(3)
           if (abs(yl).lt.dy.and.abs(zl).lt.dz) s_list(2)=s
        endif

        if ((x1(1).lt.-dx.and.x2(1).gt.-dx)) then
           s=(-dx-x1(1))/v(1)
           yl=x1(2)+s*v(2)
           zl=x1(3)+s*v(3)
           if (abs(yl).lt.dy.and.abs(zl).lt.dz) s_list(1)=s
        endif



        if ((x1(2).gt.dy.and.x2(2).lt.dy)) then
           s=(dy-x1(2))/v(2)
           xl=x1(1)+s*v(1)
           zl=x1(3)+s*v(3)
           if (abs(xl).lt.dx.and.abs(zl).lt.dz) s_list(1)=s
        endif

        if ((x1(2).lt.dy.and.x2(2).gt.dy)) then
           s=(dy-x1(2))/v(2)
           xl=x1(1)+s*v(1)
           zl=x1(3)+s*v(3)
           if (abs(xl).lt.dx.and.abs(zl).lt.dz) s_list(2)=s
        endif

        if ((x1(2).gt.-dy.and.x2(2).lt.-dy)) then
           s=(-dy-x1(2))/v(2)
           xl=x1(1)+s*v(1)
           zl=x1(3)+s*v(3)
           if (abs(xl).lt.dx.and.abs(zl).lt.dz) s_list(2)=s
        endif

        if ((x1(2).lt.-dy.and.x2(2).gt.-dy)) then
           s=(-dy-x1(2))/v(2)
           xl=x1(1)+s*v(1)
           zl=x1(3)+s*v(3)
           if (abs(xl).lt.dx.and.abs(zl).lt.dz) s_list(1)=s
        endif



        if ((x1(3).gt.dz.and.x2(3).lt.dz)) then
           s=(dz-x1(3))/v(3)
           xl=x1(1)+s*v(1)
           yl=x1(2)+s*v(2)
           if (abs(xl).lt.dx.and.abs(yl).lt.dy) s_list(1)=s
        endif

        if ((x1(3).lt.dz.and.x2(3).gt.dz)) then
           s=(dz-x1(3))/v(3)
           xl=x1(1)+s*v(1)
           yl=x1(2)+s*v(2)
           if (abs(xl).lt.dx.and.abs(yl).lt.dy) s_list(2)=s
        endif

        if ((x1(3).gt.-dz.and.x2(3).lt.-dz)) then
           s=(-dz-x1(3))/v(3)
           xl=x1(1)+s*v(1)
           yl=x1(2)+s*v(2)
           if (abs(xl).lt.dx.and.abs(yl).lt.dy) s_list(2)=s
        endif

        if ((x1(3).lt.-dz.and.x2(3).gt.-dz)) then
           s=(-dz-x1(3))/v(3)
           xl=x1(1)+s*v(1)
           yl=x1(2)+s*v(2)
           if (abs(xl).lt.dx.and.abs(yl).lt.dy) s_list(1)=s
        endif

        if (s_list(1).ge.0.0) then
          in_out_list(1)=1
          in_out_list(2)=-1
          n_entries=2
        endif


      elseif(in_1.and.(.not.in_2)) then

        v(1)=x2(1)-x1(1)
        v(2)=x2(2)-x1(2)
        v(3)=x2(3)-x1(3)

        in_out_list(1)=-1

        n_entries=1


        if ((x1(1).lt.dx.and.x2(1).gt.dx)) then
           s=(dx-x1(1))/v(1)
           yl=x1(2)+s*v(2)
           zl=x1(3)+s*v(3)
           if (abs(yl).lt.dy.and.abs(zl).lt.dz) s_list(1)=s
        endif

        if ((x1(1).gt.-dx.and.x2(1).lt.-dx)) then
           s=(-dx-x1(1))/v(1)
           yl=x1(2)+s*v(2)
           zl=x1(3)+s*v(3)
           if (abs(yl).lt.dy.and.abs(zl).lt.dz) s_list(1)=s
        endif



        if ((x1(2).lt.dy.and.x2(2).gt.dy)) then
           s=(dy-x1(2))/v(2)
           xl=x1(1)+s*v(1)
           zl=x1(3)+s*v(3)
           if (abs(xl).lt.dx.and.abs(zl).lt.dz) s_list(1)=s
        endif

        if ((x1(2).gt.-dy.and.x2(2).lt.-dy)) then
           s=(-dy-x1(2))/v(2)
           xl=x1(1)+s*v(1)
           zl=x1(3)+s*v(3)
           if (abs(xl).lt.dx.and.abs(zl).lt.dz) s_list(1)=s
        endif



        if ((x1(3).lt.dz.and.x2(3).gt.dz)) then
           s=(dz-x1(3))/v(3)
           xl=x1(1)+s*v(1)
           yl=x1(2)+s*v(2)
           if (abs(xl).lt.dx.and.abs(yl).lt.dy) s_list(1)=s
        endif

        if ((x1(3).gt.-dz.and.x2(3).lt.-dz)) then
           s=(-dz-x1(3))/v(3)
           xl=x1(1)+s*v(1)
           yl=x1(2)+s*v(2)
           if (abs(xl).lt.dx.and.abs(yl).lt.dy) s_list(1)=s
        endif

      elseif(in_2.and.(.not.in_1)) then

        v(1)=x2(1)-x1(1)
        v(2)=x2(2)-x1(2)
        v(3)=x2(3)-x1(3)

        in_out_list(1)=1

        n_entries=1

        if ((x1(1).gt.dx.and.x2(1).lt.dx)) then
           s=(dx-x1(1))/v(1)
           yl=x1(2)+s*v(2)
           zl=x1(3)+s*v(3)
           if (abs(yl).lt.dy.and.abs(zl).lt.dz) s_list(1)=s
        endif

        if ((x1(1).lt.-dx.and.x2(1).gt.-dx)) then
           s=(-dx-x1(1))/v(1)
           yl=x1(2)+s*v(2)
           zl=x1(3)+s*v(3)
           if (abs(yl).lt.dy.and.abs(zl).lt.dz) s_list(1)=s
        endif



        if ((x1(2).gt.dy.and.x2(2).lt.dy)) then
           s=(dy-x1(2))/v(2)
           xl=x1(1)+s*v(1)
           zl=x1(3)+s*v(3)
           if (abs(xl).lt.dx.and.abs(zl).lt.dz) s_list(1)=s
        endif

        if ((x1(2).lt.-dy.and.x2(2).gt.-dy)) then
           s=(-dy-x1(2))/v(2)
           xl=x1(1)+s*v(1)
           zl=x1(3)+s*v(3)
           if (abs(xl).lt.dx.and.abs(zl).lt.dz) s_list(1)=s
        endif



        if ((x1(3).gt.dz.and.x2(3).lt.dz)) then
           s=(dz-x1(3))/v(3)
           xl=x1(1)+s*v(1)
           yl=x1(2)+s*v(2)
           if (abs(xl).lt.dx.and.abs(yl).lt.dy) s_list(1)=s
        endif

        if ((x1(3).lt.-dz.and.x2(3).gt.-dz)) then
           s=(-dz-x1(3))/v(3)
           xl=x1(1)+s*v(1)
           yl=x1(2)+s*v(2)
           if (abs(xl).lt.dx.and.abs(yl).lt.dy) s_list(1)=s
        endif

      endif

      return

      end






