*
* $Id: cleofit_lin.F,v 1.7 1996/03/12 14:19:52 zfiles Exp $
*
* $Log: cleofit_lin.F,v $
* Revision 1.7  1996/03/12 14:19:52  zfiles
* Updates from Anders Ryd
*
* Revision 1.6  1995/12/03 04:17:16  zfiles
* Minor bug fixes from Rob
*
* Revision 1.5  1995/11/28 19:11:40  zfiles
* Correctly fill xxx_in variables when END_TYPE=1
*
* Revision 1.4  1995/09/13  13:50:11  zfiles
* Mods from Rob. Bug fixes
*
* Revision 1.3  1995/09/05  13:13:10  zfiles
*  Remove diagnostic printout.
*
* Revision 1.2  1995/08/29  13:28:27  zfiles
* Complete new set from Rob/Anders
*
*
C
C     Code created by 
C
C     Robert Kutschke : kutschke@lns598.lns.cornell.edu
C     Anders Ryd      : ryd@charm.physics.ucsb.edu
C
C     Created : Sept 1994 -- June 1995
C     Title   : cleofit_lin
C     Purpose : This is the main fitting routine
C               This routine opperates on the variables
C               deta(i) has the current best guess at point i
C               before any information has been added. We also
C               have the information infostar(i).
C               after information has been added at the point we have 
C               detaopt(i) and info(i). (These are on the invard pass of the 
C               filter saved in the smoothing bank). Then after adding 
C               multiple scattering at i we have detaopt(i) and infostar(i) .
C               Then we transport to i-1 and have deta(i-1) and infostar(i-1)
C               This is done as a linearization around a seed eta0.
C     
C
C     Modifications:
C       Sept  09-95 ryd : Fixed a bug in the filling of the
C                         eta_out variable. (Wrong indices)
C                         Also moved the exit point for the outward
C                         pass to avoid problems with the deletion
C                         of the outer most hit on the track.
C       Nov   27-95 kutschke:
C                   When hit ordering was reversed, we forgot to
C                   change the index used to find eta_in and v_in
C                   when end_type.eq.1.  This is now fixed.
C       Dec    2-95 kutschke:
C                   Do updating on the outward pass.
C       March 11-96 ryd:
C                   Added escape from the routine if error is
C                   set after returning from klmn_trans.
C
      subroutine cleofit_lin



      implicit none

#include "klmn/klmnseq/cleofitcom.inc"
#include "klmn/klmnseq/klmncm.inc"
#include "klmn/klmnseq/mater_n.inc"
#include "klmn/klmnseq/mater_stop.inc"
#include "klmn/klmnseq/klmnctrl.inc"
#include "klmn/klmnseq/klmntrk.inc"
#include "klmn/klmnseq/klmnerror.inc"

      integer i,j,k

      logical update,updatez
    

      real deta(5),detaopt(5),eta0(5),eta(5),deta_d(5)
      real*8 veta(5,5),vetastar(5,5)
      real eta0_o(5),rstop
      external dtime

      update=.false.
      updatez=.false.

      call klmn_init(eta0,deta,vetastar)
 
      do 10,i=1,nhits

        if (.not.klmnctrl_reshit)
     +       call klmn_save_track_par(i,eta0,deta,vetastar)

        call klmn_add_hit(i,eta0,deta,vetastar,detaopt,veta)

        if (.not.update) then
          if ( eta0(1) .ne. 0.and.(i-1).gt.10.and.
     +             10*abs(detaopt(1)).lt.abs(eta0(1)) ) then
            if (veta(1,1)/(eta0(1)**2) .lt. 0.01D0 ) then
               update=.true. 
            endif
          endif
        endif

        if (update.and.10*abs(detaopt(1)).gt.abs(eta0(1)) ) then
             update=.false.
        endif

        if (update) then
          eta0(1)=eta0(1)+detaopt(1)
          eta0(2)=eta0(2)+detaopt(2)
          eta0(3)=eta0(3)+detaopt(3)
          detaopt(1)=0.0
          detaopt(2)=0.0
          detaopt(3)=0.0
          if (abs(eta0(1)).lt.0.02) then
c             print *,'Warning in update eta0(1) to small'
             detaopt(1)=eta0(1)-0.02*sign(1.0,eta0(1))
             eta0(1)=0.02*sign(1.0,eta0(1))        
          endif
        endif

        if (klmnctrl_reshit)
     +       call klmn_save_track_par(i,eta0,detaopt,veta)

C       Will transport

        if (ttype(i).eq.1) then
           call ucopy(detaopt,deta,5)
           call ucopy(veta,vetastar,2*25)
           goto 10
        endif

        if (next_type(i).eq.0) then
          if (i.eq.nhits) then
            call klmn_trans(x(i),y(i),0.0,0.0,eta0,detaopt,
     +                     veta,eta0,deta,vetastar)
            if (klmnerror_error(klmnctrl_part)) goto 999
          else
            call klmn_trans(x(i),y(i),x(i+1),y(i+1),eta0,detaopt,
     +                     veta,eta0,deta,vetastar)
            if (klmnerror_error(klmnctrl_part)) goto 999
          endif
        else
          if (next_type(i).lt.0) then
            next_type(i)=-next_type(i)
            call t_to_t_c(eta0,eta0_o,x(i),y(i),0.0,0)
            do 6,j=0,n_next_point(i)-1       
              rstop=x_stop_mat(j+next_type(i))
              call cyl_int(eta0_o(1),eta0_o(3),eta0_o(2),
     +          eta0_o(5),eta0_o(4),rstop,
     +          x_stop_mat(j+next_type(i)),
     +          y_stop_mat(j+next_type(i)))
 6          continue
          endif

          call klmn_trans(x(i),y(i),x_stop_mat(next_type(i)),
     +              y_stop_mat(next_type(i)),
     +              eta0,detaopt,veta,eta0,deta,vetastar)
          if (klmnerror_error(klmnctrl_part)) goto 999


          do 7,j=0,n_next_point(i)-2
            call klmn_trans(x_stop_mat(j+next_type(i)),
     +              y_stop_mat(j+next_type(i)),
     +              x_stop_mat(j+1+next_type(i)),
     +              y_stop_mat(j+1+next_type(i)),
     +              eta0,deta,vetastar,eta0,deta,vetastar)
            if (klmnerror_error(klmnctrl_part)) goto 999
 7        continue
          if (i.eq.nhits) then
            call klmn_trans(x_stop_mat(next_type(i)+n_next_point(i)-1),
     +              y_stop_mat(next_type(i)+n_next_point(i)-1),0.0,0.0,
     +              eta0,deta,vetastar,eta0,deta,vetastar)
            if (klmnerror_error(klmnctrl_part)) goto 999
          else
            call klmn_trans(x_stop_mat(next_type(i)+n_next_point(i)-1),
     +            y_stop_mat(next_type(i)+n_next_point(i)-1),
     +            x(i+1),y(i+1),eta0,deta,vetastar,eta0,deta,vetastar)
            if (klmnerror_error(klmnctrl_part)) goto 999
          endif
        endif     


C     Have transported

 10   continue


C     save parameters at point of closest approach


      if (end_type.eq.0) then


        do 1235,j=1,5
          do 1236,k=1,5
            v_in(j,k,klmnctrl_part)=vetastar(j,k)
1236      continue
1235    continue

        eta_in(1,klmnctrl_part)=eta0(1)+deta(1)
        eta_in(3,klmnctrl_part)=eta0(3)+deta(3)
        eta_in(2,klmnctrl_part)=eta0(2)+deta(2)
        eta_in(5,klmnctrl_part)=eta0(5)+deta(5)
        eta_in(4,klmnctrl_part)=eta0(4)+deta(4)

        chisq_in(klmnctrl_part)=chisq      

        xo_in=0.0
        yo_in=0.0
        zo_in=0.0

      elseif(end_type.eq.1) then

        do 456,i=1,5
          do 457,j=1,5
            v_in(i,j,klmnctrl_part)=veta_s(i,j,nhits)
457       continue
456     continue

        call ucopy(eta_s(1,nhits),eta_in(1,klmnctrl_part),5)        

        chisq_in(klmnctrl_part)=chisq      

        xo_in=x(nhits)
        yo_in=y(nhits)
        zo_in=0.

      else
         print *,'Wrong end_type'
      endif

C     Done with inwards pass

      if (.not.klmnctrl_smooth) goto 999

      call klmn_init_smooth(eta0,deta,vetastar)

      update = .false.
      do 40,i=nhits,1,-1


         call klmn_smooth_d(i,eta0,deta,vetastar,eta,veta)

         call klmn_save_smooth_par(i,eta,veta)       

         if (klmnctrl_res) then
             if (abs((eta(1)-eta0(1))/eta0(1)).gt.0.5) then
c                 print *,'Warning smoothing has gone wrong!'
                 eta(1)=eta0(1)
                 eta(2)=eta0(2)
                 eta(3)=eta0(3)
                 eta(4)=eta0(4)
                 eta(5)=eta0(5)
              endif
            call calc_res(i,eta,veta)
         endif

         call klmn_add_hit(i,eta0,deta,vetastar,detaopt,veta)

        if (.not.update) then
          if ( eta0(1) .ne. 0.and.(nhits-i).gt.10.and.
     +             10*abs(detaopt(1)).lt.abs(eta0(1)) ) then
            if (veta(1,1)/(eta0(1)**2) .lt. 0.01D0 ) then
               update=.true. 
            endif
          endif
        endif

        if (update.and.10*abs(detaopt(1)).gt.abs(eta0(1)) ) then
             update=.false.
        endif

        if (update) then
          eta0(1)=eta0(1)+detaopt(1)
          eta0(2)=eta0(2)+detaopt(2)
          eta0(3)=eta0(3)+detaopt(3)
          detaopt(1)=0.0
          detaopt(2)=0.0
          detaopt(3)=0.0
          if (abs(eta0(1)).lt.0.02) then
c             print *,'Warning in update eta0(1) to small'
             detaopt(1)=eta0(1)-0.02*sign(1.0,eta0(1))
             eta0(1)=0.02*sign(1.0,eta0(1))        
          endif
        endif

         if (i.eq.1) goto 40

         if (ttype(i-1).eq.1) then
           call ucopy(detaopt,deta,5)
           call ucopy(veta,vetastar,2*25)
           goto 40
         endif

         if (next_type(i-1).eq.0) then
            call klmn_trans(x(i),y(i),x(i-1),y(i-1),eta0,detaopt,
     +                     veta,eta0,deta,vetastar)
            if (klmnerror_error(klmnctrl_part)) goto 999
         else

           call klmn_trans(x(i),y(i),
     +              x_stop_mat(next_type(i-1)+n_next_point(i-1)-1),
     +              y_stop_mat(next_type(i-1)+n_next_point(i-1)-1),
     +              eta0,detaopt,veta,eta0,deta,vetastar)
            if (klmnerror_error(klmnctrl_part)) goto 999


           do 47,j=n_next_point(i-1)-1,1,-1

             call klmn_trans(x_stop_mat(j+next_type(i-1)),
     +              y_stop_mat(j+next_type(i-1)),
     +              x_stop_mat(j-1+next_type(i-1)),
     +              y_stop_mat(j-1+next_type(i-1)),
     +              eta0,deta,vetastar,eta0,deta,vetastar)
            if (klmnerror_error(klmnctrl_part)) goto 999

 47        continue

           call klmn_trans(x_stop_mat(next_type(i-1)),
     +            y_stop_mat(next_type(i-1)),
     +            x(i-1),y(i-1),eta0,deta,vetastar,eta0,deta,vetastar)
            if (klmnerror_error(klmnctrl_part)) goto 999

         endif     


 40   continue

C     Done with outward pass
C     eta0 contains the seed
C     detaopt is the track 
C     veta is error matrix

      call t_to_t(eta0,detaopt,veta,eta0,detaopt,veta,x(1),y(1),0.0,-1)

      do 1123,i=1,5
        do 1124,j=1,5
          v_out(i,j)=veta(i,j)
1124    continue
1123  continue  
       
      eta_out(1)=eta0(1)+detaopt(1)
      eta_out(2)=eta0(2)+detaopt(2)
      eta_out(3)=eta0(3)+detaopt(3)
      eta_out(4)=eta0(4)+detaopt(4)
      eta_out(5)=eta0(5)+detaopt(5)
     

999   continue

      return
 
      end
















