*
* $Id: cleosvxder.F,v 1.3 1995/11/28 19:12:24 zfiles Exp $
*
* $Log: cleosvxder.F,v $
* Revision 1.3  1995/11/28 19:12:24  zfiles
* Limit the number of iterations
*
* Revision 1.2  1995/08/29  13:28:28  zfiles
* Complete new set from Rob/Anders
*
*
C
C
C     Code created by 
C
C     Robert Kutschke : kutschke@lns598.lns.cornell.edu
C     Anders Ryd      : ryd@charm.physics.ucsb.edu
C
C     Created : Sept 1994 -- June 1995
C     Title   :cleosvxder 
C     Purpose :This routine calculates the derivative of a position
C              on a plane with respect to the 5 track parameters.
C     
C
C     Modifications:
C     Nov 27-95 kutschke:
C               Limit the number of iterations.  This is very rare
C               and we just let the routine return junk.
C
C     
      subroutine cleosvxder(k,d0,phi,z,t,x0,v0,u0,derv,dpmin)
 
      implicit none

      

      integer i,n_it
      real k,d0,phi,z,t,psi,x0(3),v0(3),u0(3)
      real derv(5),dpmin     

      real n(3)
      real sphi,cphi,q,rho,sqpsi,cqpsi
      real xt,yt,zt,c2,s,dx,dy,dz,ax,ay,az,vx,vy,vz
      real u,v,dv,du,vpsi(3),vk(3),vd(3),vphi(3)
      real vz0(3),vt(3),nvpsiinv,v0vpsi,inv2k,inv2k2
      real dpsidk,dpsidd,dpsidphi,dpsidz,dpsidt,invcsclambda

      n_it=0

      call cross(v0,u0,n)

      inv2k=1.0/(2*k)
      inv2k2=inv2k/k

      psi=0.0

      sphi=sin(phi)
      cphi=cos(phi)

      q=sign(1.0,k)

      rho=abs(inv2k)

      invcsclambda=1.0/sqrt(1.0+t**2)

      vz=t*invcsclambda


 10   sqpsi=sin(phi+q*psi)
      cqpsi=cos(phi+q*psi)

      xt=-d0*sphi+(sqpsi-sphi)*inv2k
      yt=d0*cphi+(cphi-cqpsi)*inv2k
      zt=z+t*rho*psi

      vx=cqpsi*invcsclambda

      vy=sqpsi*invcsclambda


c     calculate the s, the distance along (vx,vy,vz)

      s=(n(1)*(xt-x0(1))+n(2)*(yt-x0(2))+n(3)*(zt-x0(3)))/
     +       (n(1)*vx+n(2)*vy+n(3)*vz)

      psi=psi-s*abs(2*k)*invcsclambda

      n_it=n_it+1

      if (n_it.eq.50) goto 11

      if (abs(2*s**2*inv2k).gt.0.000001) goto 10


11    sqpsi=sin(phi+q*psi)
      cqpsi=cos(phi+q*psi)

      xt=-d0*sphi+(sqpsi-sphi)*inv2k
      yt=d0*cphi+(cphi-cqpsi)*inv2k
      zt=z+t*rho*psi


      dpmin=v0(1)*(xt-x0(1))+v0(2)*(yt-x0(2))+v0(3)*(zt-x0(3))

      vpsi(1)=cqpsi*rho

      vpsi(2)=sqpsi*rho

      vpsi(3)=t*rho

      vk(1)=-(sqpsi-sphi)*inv2k2
      vk(2)=-(cphi-cqpsi)*inv2k2
      vk(3)=-t*q*psi*inv2k2

      vd(1)=-sphi
      vd(2)=cphi

      vphi(1)=-yt
      vphi(2)=xt

      vt(3)=rho*psi


      nvpsiinv=1.0/(n(1)*vpsi(1)+n(2)*vpsi(2)+n(3)*vpsi(3))

      dpsidk=(n(1)*vk(1)+n(2)*vk(2)+n(3)*vk(3))*nvpsiinv
      dpsidd=(n(1)*vd(1)+n(2)*vd(2))*nvpsiinv
      dpsidphi=(n(1)*vphi(1)+n(2)*vphi(2))*nvpsiinv
      dpsidz=n(3)*nvpsiinv
      dpsidt=n(3)*vt(3)*nvpsiinv

      
      v0vpsi=v0(1)*vpsi(1)+v0(2)*vpsi(2)+v0(3)*vpsi(3)

      derv(1)=(v0(1)*vk(1)+v0(2)*vk(2)+v0(3)*vk(3))-dpsidk*v0vpsi
      derv(3)=(v0(1)*vd(1)+v0(2)*vd(2))-dpsidd*v0vpsi
      derv(2)=(v0(1)*vphi(1)+v0(2)*vphi(2))-dpsidphi*v0vpsi
      derv(5)=v0(3)-dpsidz*v0vpsi
      derv(4)=v0(3)*vt(3)-dpsidt*v0vpsi


      return

      end












