*
* $Id: de.F,v 1.8 1996/04/23 23:56:43 zfiles Exp $
*
* $Log: de.F,v $
* Revision 1.8  1996/04/23 23:56:43  zfiles
* Comments in previous version are wrong!
* True comment:
*      Protect against de.lt.0.
*
* Revision 1.7  1996/04/23 06:56:41  zfiles
* More mods from Rob.
*
* Revision 1.6  1996/04/08 14:41:34  zfiles
* Fix buggy treatment of stopping particles
*
* Revision 1.5  1996/03/03 00:27:22  zfiles
* Removed warning messages. Add comments.
*
* Revision 1.4  1995/09/05 13:13:46  zfiles
*  Limit step length to neighbourhood of bin size.
*  Use in/out switch in stoprg calcuation.
*
* Revision 1.3  1995/08/29  17:35:09  zfiles
* Compatibility with AIX
*
* Revision 1.2  1995/08/29  13:28:30  zfiles
* Complete new set from Rob/Anders
*
*
C 
C     Code created by 
C
C     Robert Kutschke : kutschke@lns598.lns.cornell.edu
C     Anders Ryd      : ryd@charm.physics.ucsb.edu
C
C     Created : Sept 1994 -- June 1995
C     Title   : de
C     Purpose : Calculates the mean energy loss in a distance of step_s.
C
C               This code is lifted from the GEANT 3.14 routines
C               gtmuon, gthadr, and gtelec.
C
C               There are three ways in which the code can decide that
C               a particle has stopped.  These can only be reached
C               when a particle is being traced outward ( klmnctrl_in=-1).
C               In these cases de is limited as follows:
C                a) if e.lt.emin, then de=0.
C                b) otherwise de=e-emin; therefore the caller will compute
C                   e_new = e-de = emin.
C
C     Modifications:
C     Rob Kutschke April 6/96.
C        Catch iekbin.lt.1 and treat it as a stopped particle.
C        Set de=0. if any of the traps for stopped particles
C        would otherwise force de to be negative.
C
      real function de(e,n_mater,step_s,beta)

      implicit none

#include "klmn/klmnseq/mater_prop.inc"
#include "klmn/klmnseq/cleofitcom.inc"
#include "klmn/klmnseq/klmnctrl.inc"
#include "klmn/klmnseq/klmntrk.inc"

      real e,step_s,beta,de1,e1
 
      real gekrat,gekrt1

      real*8 xcoef1,xcoef2,xcoef3,e_kin,step,stoprg,stopmx
      real*8 stopp,rmass,ekipr,ek,step_left
 
      real*8 epsmac
      parameter ( epsmac  = 1.e-6 )

C     Choose this value so that the minimum total energy of a proton
C     is significantly different than its rest mass ( in single precision).
      real emin
      parameter ( emin = 0.0001 )

      integer lekbin,n_mater,iekbin,part_type


C     proton mass.
      real*8 pmass 
      parameter ( pmass = 0.9382723128 ) 

C     save previous value so that we can save time in
C     finding entry in energyloss table
      save iekbin
      data iekbin / 51 /

C
C       Reminder of what the particle numbers means:
C
C       klmnctrl_part  part_type
C         1 = e         1=e-
C         2 = mu        2=e+
C         3 = pi        3=mu
C         4 = ka        4=pi
C         5 = pr        5=ka
C                       6=pr
C
      integer nloop

c      print *,'In de: e,n_mater,step_s,beta,mass',
c     +         e,n_mater,step_s,beta,klmnctrl_mass


      if (klmnctrl_part.eq.1) then
        if (klmnctrl_q.lt.0) then
          part_type=1
        else
          part_type=2
        endif
      else
        part_type=klmnctrl_part+1
      endif 


      step=step_s
      stepsum = stepsum+step


C     Limit the kinetic energy:
C       lower bound - lekbin must lie in the table ( cut off at
C                     10KeV ).  
C       upper bound - CLEO has no real tracks with kinetic energy
C                     above 10. GeV.  ( upper bound of table is
C                     far above this ).

      e_kin=amax1(amin1(e,10.),elow_t(2))
      ek=e_kin    

      rmass=pmass/dble(klmnctrl_mass)
      if (part_type.gt.3) ek=ek*rmass

C     See if we have to find new entry in table
      if (ek.lt.elow_t(iekbin).or.ek.gt.elow_t(iekbin+1)) then
        iekbin=10.0*log10(ek)+51.0
      endif

      gekrat=(ek-elow_t(iekbin))/(elow_t(iekbin+1)-elow_t(iekbin))
      gekrat=min(1.0,max(gekrat,0.0))
      gekrt1=1.0-gekrat

C     If the step length is unrealisticly long this code
C     will divide it into smaller steps.

      if (step*par_eloss(part_type,iekbin,n_mater).gt.
     +    elow_t(iekbin+1)-elow_t(iekbin)) then
c        print *,"Reducing step length in de.F", part_type, e_kin
        step_left=step
        de=0.0
        e1=e_kin
        nloop = 0
10      continue
        nloop = nloop + 1
        step=(elow_t(iekbin+1)-elow_t(iekbin))/
     +    par_eloss(part_type,iekbin,n_mater)

        if (step.gt.step_left) step=step_left

        de1=(gekrt1*par_eloss(part_type,iekbin,n_mater)
     +      +gekrat*par_eloss(part_type,iekbin+1,n_mater))
     +      *step

        de=de+de1
        e1=e1+dfloat(klmnctrl_in)*de1
        ek=e1

C       Particle has stopped. 
        if ( ek .le. 0. ) then
           de = (e-emin)
           if ( de .lt. 0 ) de = 0.
           goto 999
        endif

        step_left=step_left-step
        step=step_left

        if (part_type.gt.3) ek=ek*rmass

        if (ek.lt.elow_t(iekbin).or.ek.gt.elow_t(iekbin+1)) then
          iekbin=10.0*log10(ek)+51.0
        endif

C       Particle has slowed down so that it has dropped out of
C       the table.  Treat it as stopped.
        if (iekbin .lt. 1 ) then
           de = (e-emin)
           if ( de .lt. 0 ) de = 0.
           iekbin = 1
           goto 999
        endif

        gekrat=(ek-elow_t(iekbin))/
     +         (elow_t(iekbin+1)-elow_t(iekbin))
        gekrat=min(1.0,max(gekrat,0.0))
        gekrt1=1.0-gekrat

        if (step_left.le.1.d-6) goto 999

        if (nloop .gt. 50 ) then
c          print *, 'Infinite loop in de.F ', nloop
          goto 999
        endif

        goto 10

      endif

C     check if fast energy loss is good enogh
      if (beta.gt.0.8) then
         de=(gekrt1*par_eloss(part_type,iekbin,n_mater)
     +      +gekrat*par_eloss(part_type,iekbin+1,n_mater))
     +      *step
         goto 999
      endif

      if (gekrat.lt.0.7) then
        lekbin=iekbin-1
      else
        lekbin=iekbin
      endif


      xcoef1=par_xcoef1(part_type,lekbin,n_mater)
      xcoef2=par_xcoef2(part_type,lekbin,n_mater)
      xcoef3=par_xcoef3(part_type,lekbin,n_mater)

      if (xcoef1.eq.0.0.and.
     +    xcoef2.eq.0.0.and.
     +    xcoef3.eq.0.0) then
         de=0.0
         goto 999
       endif

C     Here we branch to the treatment of the leptons
C     part_type<=3
      if (part_type.le.3) then
        if (xcoef1.ne.0.0) then
          stopmx=-xcoef2+sign(1.0D0,xcoef1)*
     +        sqrt(xcoef2**2-(xcoef3-e_kin/xcoef1))
          stoprg=stopmx+dfloat(klmnctrl_in)*step
          de=-(e_kin-xcoef1*(xcoef3+stoprg*(2.0*xcoef2+stoprg)))
        else
          stopmx=-(xcoef3-e_kin)/xcoef2 
          stoprg=stopmx+step
          de=-(e_kin-xcoef2*stoprg-xcoef3)
        endif

C     Now treat the hadrons
      elseif (part_type.gt.3) then
        if(xcoef1.ne.0) then
          stopp=-xcoef2+sign(1.D0,xcoef1)*
     +        sqrt(xcoef2**2-(xcoef3-e_kin*rmass/xcoef1))
          stoprg=stopp+step*rmass
          ekipr=xcoef1*(xcoef3+stoprg*(2.0*xcoef2+stoprg))
        else
          stopp=-(xcoef3-e_kin*rmass)/xcoef2
          stoprg=stopp+dfloat(klmnctrl_in)*step*rmass
          ekipr=xcoef2*stoprg+xcoef3
        endif
        de=-(e_kin-ekipr/rmass)

        if (de.le.5.*e_kin*epsmac) then
            de=(gekrt1*par_eloss(part_type,iekbin,n_mater)
     +            +gekrat*par_eloss(part_type,iekbin+1,n_mater))
     +             *step
        endif

      endif

C     Check for a stopped particle.
      if ( klmnctrl_in .eq. -1 ) then
         if ( e-de .le. 0. ) de = e-emin
      endif

999   continue     
      if ( de .lt. 0. ) de = 0.

      return
      end
