*
* $Id: klmn_arc.F,v 1.2 1995/11/28 19:19:40 zfiles Exp $
*
* $Log: klmn_arc.F,v $
* Revision 1.2  1995/11/28 19:19:40  zfiles
* Id and Log added
*
*
C     Code created by 
C
C     Anders Ryd      : ryd@charm.physics.ucsb.edu
C
C     Created : Sept 27 1995
C     Title   : klmn_arc
C     Purpose : To calculate the arc-length of the track
C               from the point of closest impact to the 
C               origin and out to the ToF detector. The
C               Time of Flight is modeled as a cylinder
C               with radius rmax and a half length zend.
C     
C
C     Modifications:
C
C
C
C
      subroutine klmn_arc

      implicit none 

      real arc

#include "klmn/klmnseq/cleofitcom.inc"
#include "klmn/klmnseq/klmncm.inc"
#include "klmn/klmnseq/mater_n.inc"
#include "klmn/klmnseq/mater_stop.inc"
#include "klmn/klmnseq/klmnctrl.inc"
#include "klmn/klmnseq/klmntrk.inc"

       integer i,n
       real xp(3),v(3),s,psi,dpsi,xstop(3)
       real s1,s2,ss,r,dist,psi1,xv,x2

       real zend,rmax

       integer ierr
#include "seq/clutil/mesrep.inc"

c       print *,'In klmn_arc'

       zend=1.2040
       rmax=0.9680

       arc=0.0

       psi=0.0
       n=0
c         print *,'eta:',eta_s(1,nhits),eta_s(2,nhits),
c     +   eta_s(3,nhits),eta_s(4,nhits),eta_s(5,nhits)

2        call klmn_xv(eta_s(1,nhits),psi,xp,v)
         xp(1)=xp(1)+x_s(nhits)
         xp(2)=xp(2)+y_s(nhits)
         xp(3)=xp(3)+z_s(nhits)
c         print *,'xp:',xp(1),xp(2),xp(3)
c         print *,'v :',v(1),v(2),v(3)
         ss=-(xp(1)*v(1)+xp(2)*v(2)+xp(3)*v(3)) 
         dpsi=ss*abs(2*eta_s(1,nhits))/sqrt(1+eta_s(4,nhits)**2)
         psi=psi+dpsi
c         print *,'1:psi,dpsi,ss:',psi,dpsi,ss
         n=n+1
c         if (n.eq.20) print *,'1:n=20'
         if (n.eq.20) then
            arc=-100.0
            goto 999
         endif           
         if (abs(ss).gt.0.0001) goto 2

         call klmn_xv(eta_s(1,nhits),psi,xstop,v)
         xstop(1)=xstop(1)+x_s(nhits)
         xstop(2)=xstop(2)+y_s(nhits)
         xstop(3)=xstop(3)+z_s(nhits)
       

       do 10,i=nhits,2,-1

c    find turning angle back to xstop.

         psi=0.0
         n=0
c         print *,'eta:',eta_s(1,i),eta_s(2,i),
c     +          eta_s(3,i),eta_s(4,i),eta_s(5,i)
3        call klmn_xv(eta_s(1,i),psi,xp,v)
         xp(1)=xp(1)+x_s(i)
         xp(2)=xp(2)+y_s(i)
         xp(3)=xp(3)+z_s(i)
         ss=-((xp(1)-xstop(1))*v(1)+(xp(2)-xstop(2))*v(2)+
     +                 (xp(3)-xstop(3))*v(3)) 
         dpsi=ss*abs(2*eta_s(1,i))/sqrt(1+eta_s(4,i)**2)
         psi=psi+dpsi
c         print *,'2:psi,dpsi,ss:',psi,dpsi,ss
         n=n+1
c         if (n.eq.20) print *,'2:n=20'
         if (n.eq.20) then
            arc=-100.0
            goto 999
         endif           
         if (abs(ss).gt.0.0001) goto 3

         psi1=psi

         dist=0.5*sqrt((x_s(i)-x_s(i-1))**2+
     +             (y_s(i)-y_s(i-1))**2)

         psi=dist*abs(2*eta_s(1,i))/sqrt(1+eta_s(4,i)**2)

         if (psi1.gt.0.0) then
           psi=-psi
         endif

         arc=arc+(psi1-psi)*sqrt(1+eta_s(4,i)**2)/abs(2*eta_s(1,i))

         call klmn_xv(eta_s(1,i),psi,xstop,v)
         xstop(1)=xstop(1)+x_s(i)
         xstop(2)=xstop(2)+y_s(i)
         xstop(3)=xstop(3)+z_s(i)
       
       

10     continue


         psi=0.0
         n=0
c         print *,'eta:',eta_s(1,1),eta_s(2,1),
c     +          eta_s(3,1),eta_s(4,1),eta_s(5,1)
4        call klmn_xv(eta_s(1,1),psi,xp,v)
         xp(1)=xp(1)+x_s(1)
         xp(2)=xp(2)+y_s(1)
         xp(3)=xp(3)+z_s(1)
         ss=-((xp(1)-xstop(1))*v(1)+(xp(2)-xstop(2))*v(2)+
     +                 (xp(3)-xstop(3))*v(3)) 
         dpsi=ss*abs(2*eta_s(1,1))/sqrt(1+eta_s(4,1)**2)
c         print *,'3:psi,dpsi,ss:',psi,dpsi,ss
         psi=psi+dpsi
         n=n+1
c         if (n.eq.20) print *,'3:n=20'
         if (n.eq.20) then
            arc=-100.0
            goto 999
         endif           
         if (abs(ss).gt.0.0001) goto 4

         psi1=psi


c      now we have to do the last step.
C      find the intercept with the z or r. 

c      first find the z intercept since it is easier

       if  (eta_s(4,1).lt.0.0)  zend=-zend

       psi=(zend-eta_s(5,1))*abs(2*eta_s(1,1))/eta_s(4,1)

       call klmn_xv(eta_s(1,1),psi,xp,v)

       r=sqrt((xp(1)+x_s(1))**2+(xp(2)+y_s(1))**2)

       if (r.gt.rmax.or.abs(psi).gt.1.0) then

c        print *,'Hit in the barrel!'

c       have to find intercept with circle!

         psi=0.0
         n=0
c         print *,'eta:',eta_s(1,1),eta_s(2,1),
c     +          eta_s(3,1),eta_s(4,1),eta_s(5,1)
6        call klmn_xv(eta_s(1,1),psi,xp,v)
         xp(1)=xp(1)+x_s(1)
         xp(2)=xp(2)+y_s(1)
         xp(3)=xp(3)+z_s(1)
         xv=xp(1)*v(1)+xp(2)*v(2)+xp(3)*v(3)
         x2=xp(1)*xp(1)+xp(2)*xp(2)+xp(3)*xp(3)
         if ( rmax**2-x2+xv**2 .lt. 0 ) then
            arc = -100.
            goto 999
         endif
         s1=-xv+sqrt(rmax**2-x2+xv**2)
         s2=-xv-sqrt(rmax**2-x2+xv**2)
         if (abs(s1).lt.abs(s2)) then 
           ss=s1
         else
           ss=s2
         endif
         dpsi=ss*abs(2*eta_s(1,1))/sqrt(1+eta_s(4,1)**2)
         psi=psi+dpsi
c         print *,'4:psi,dpsi,ss:',psi,dpsi,ss
         n=n+1
c         if (n.eq.20) print *,'4:n=20'
         if (n.eq.20) then
            arc=-100.0
            goto 999
         endif           
         if (abs(ss).gt.0.0001) goto 6

       else

c        print *,'Hit in the endcap!'

       endif

       arc=arc+(psi1-psi)*sqrt(1+eta_s(4,1)**2)/abs(2*eta_s(1,1))

       stepsum=abs(arc)

999    continue

c       print *,'arc:',arc

c       print *,'eta:',eta_s(1,nhits),eta_s(2,nhits),
c     +        eta_s(3,nhits),eta_s(4,nhits),eta_s(5,nhits)

c       print *,'Out klmn_arc'

       return

       end
      





