*
* $Id: klmn_trans.F,v 1.7 1996/04/23 23:53:52 zfiles Exp $
*
* $Log: klmn_trans.F,v $
* Revision 1.7  1996/04/23 23:53:52  zfiles
* Comments in previous version are wrong!
* True comment:
*      Protect against p_to=0.0.
*
* Revision 1.6  1996/04/23 06:56:43  zfiles
* More mods from Rob.
*
* Revision 1.5  1996/03/19 16:25:55  zfiles
* Comment out code for non-uniform magnetic field
*
* Revision 1.4  1996/03/12 14:19:54  zfiles
* Updates from Anders Ryd
*
* Revision 1.3  1995/12/03 04:17:17  zfiles
* Minor bug fixes from Rob
*
* Revision 1.2  1995/08/29  13:28:50  zfiles
* Complete new set from Rob/Anders
*
*
C 
C     Code created by 
C
C     Robert Kutschke : kutschke@lns598.lns.cornell.edu
C     Anders Ryd      : ryd@charm.physics.ucsb.edu
C
C     Created : Sept 1994 -- June 1995
C     Title   : klmn_trans
C     Purpose : To transport trackparameters. The effects of material
C               is incorporated here. Call the routine book_mater to
C               find how much material the track is passing through.
C               t_to_t transport paramters through vacuum.
C     
C
C     Modifications:
C      Dec     2-95 kutschke:
C                   Bug fix in the dedx for outward pass.  Sign change
C                   was done twice!
C      March 11-96  ryd:
C                   Add escape from the routine if error is set
C                   after returning from book_mater.
C      March 18-96  ryd:
C                   added - commented out - code that will do corrections
C                   for non-uniform magnetic field. This code has been 
C                   tested in the BaBar frame-work. This is the only 
C                   routine that needs to be modified for non-uniform 
C                   magnetic field.
C      April 19-96  kutschke:
C                   Protect against p_to going to zero.
C



      subroutine klmn_trans(x_fr,y_fr,x_to,y_to,eta_fr,deta_fr,
     +              veta_fr,eta_to,deta_to,veta_to)

      implicit none
      
      real x_fr,y_fr,x_to,y_to,sc_fac,sc_fac1,sc_fac2,re,rk,rk2

      real eta_fr(5),eta_to(5),deta_fr(5),deta_to(5),ddk
      real*8  veta_fr(5,5),veta_to(5,5)
      real   eta_fr_s(5)

     
#include "klmn/klmnseq/cleofitcom.inc"
#include "klmn/klmnseq/mater_n.inc"
#include "klmn/klmnseq/klmnctrl.inc"
#include "klmn/klmnseq/klmn_par.inc"
#include "klmn/klmnseq/klmnerror.inc"

      real eta_to_trans(5),deta_to_trans(5),e_fr,e_to,p_to
      real*8  veta_to_trans(5,5)
      real dk,v2phi,v2t,v2k,vkt,tsqrt,p,deltae

      integer j,k

C     uncomment below for non-uniform magnetic field.
C      integer kmater,chg
C      real dt,dphi,dkappa
C      real dp(3),bfld(3),dx(3),xm(3)


      eta_fr_s(1)=eta_fr(1)
      eta_fr_s(2)=eta_fr(2)
      eta_fr_s(3)=eta_fr(3)
      eta_fr_s(4)=eta_fr(4)
      eta_fr_s(5)=eta_fr(5)


      call t_to_t(eta_fr,deta_fr,veta_fr,
     +         eta_to_trans,deta_to_trans,veta_to,
     +         x_fr-x_to,y_fr-y_to,0.0,0)

c      print *,'Is checking veta_to in klmn_trans'       
c      call check_var(veta_to,'point 2')

       tsqrt=sqrt(1.0+eta_fr(4)**2)

       p=klmn_a*tsqrt/abs(eta_fr(1))

       e_fr=sqrt(klmnctrl_mass**2+p**2)


       eta_to(1)=eta_to_trans(1)
       eta_to(2)=eta_to_trans(2)
       eta_to(3)=eta_to_trans(3)
       eta_to(4)=eta_to_trans(4)
       eta_to(5)=eta_to_trans(5)

       call book_mater(eta_fr_s,x_fr,y_fr,0.0,eta_to,
     +           x_to,y_to,0.0,p,e_fr,deltae,
     +           v2phi,v2t,v2k,vkt,ddk)

       if (klmnerror_error(klmnctrl_part)) goto 999

       e_to=e_fr+klmnctrl_in*deltae
       p_to = sqrt(amax1(e_to**2-klmnctrl_mass**2,0.0001))

       dk=-klmn_a*tsqrt*
     +          (1.0/p_to-1.0/p)

       if (eta_fr(1).gt.0.0) dk=-dk

       eta_to(1)=eta_to(1)+dk



       deta_to(1)=deta_to_trans(1)
       deta_to(2)=deta_to_trans(2)
       deta_to(3)=deta_to_trans(3)
       deta_to(4)=deta_to_trans(4)
       deta_to(5)=deta_to_trans(5)


       re=e_to/e_fr
       rk=eta_to(1)/(eta_to(1)-dk)
       rk2=rk*rk

       sc_fac1=re*rk*rk2
       sc_fac2=eta_to(4)*eta_to(1)*(1.0-re*rk2)/(1.0+eta_to(4)**2)


        veta_to(1,1)=veta_to(1,1)*sc_fac1**2+
     +               veta_to(1,4)*2*sc_fac1*sc_fac2+
     +               veta_to(4,4)*sc_fac2**2+v2k+ddk
        veta_to(1,2)=veta_to(1,2)*sc_fac1+veta_to(4,2)*sc_fac2
        veta_to(2,1)=veta_to(1,2)
        veta_to(1,3)=veta_to(1,3)*sc_fac1+veta_to(4,3)*sc_fac2
        veta_to(3,1)=veta_to(1,3)
        veta_to(1,5)=veta_to(1,5)*sc_fac1+veta_to(4,5)*sc_fac2
        veta_to(5,1)=veta_to(1,5)
        veta_to(1,4)=veta_to(1,4)*sc_fac1+veta_to(4,4)*sc_fac2+vkt
        veta_to(2,2)=veta_to(2,2)+v2phi
        veta_to(4,1)=veta_to(1,4)
        veta_to(4,4)=veta_to(4,4)+v2t


c        print *,'Is checking veta_to in klmn_trans after update'       
c        call check_var(veta_to,'point 3')


C     Comment out code below to get magnetic field correction
c     Will do the magnetic field correction
c
c      if (klmnctrl_buildmater) then
c        kmater=nmater_n
c      else
c        kmater=imater_n
c      endif
c
c      xm(1)=0.5*(x_mater_n(1,kmater)+x2_mater_n(1,kmater))      
c      xm(2)=0.5*(x_mater_n(2,kmater)+x2_mater_n(2,kmater))      
c      xm(3)=0.5*(x_mater_n(3,kmater)+x2_mater_n(3,kmater))      
c
c      dx(1)=x_mater_n(1,kmater)-x2_mater_n(1,kmater)
c      dx(2)=x_mater_n(2,kmater)-x2_mater_n(2,kmater)
c      dx(3)=x_mater_n(3,kmater)-x2_mater_n(3,kmater)
c
c      call a routine 'gufld' that returns the magnetic field "bfld"
c      in T at the point "xm" in meters.
c
c      call gufld(xm,bfld)
c
c      Now find the perturbation in the magnetic field
c
c      bfld(3)=bfld(3)-klmn_bfield
c
c      if (eta_to(1).lt.0.0) then
c        chg=-1
c      else
c        chg=1
c      endif
c
c      Calculate the momentum change due to the residual
c      magnetic field
c
c      dp(1)=0.3*chg*(bfld(2)*dx(3)-bfld(3)*dx(2))
c      dp(2)=0.3*chg*(bfld(3)*dx(1)-bfld(1)*dx(3))
c      dp(3)=0.3*chg*(bfld(1)*dx(2)-bfld(2)*dx(1))
c
c     calculate the change in the track parameters
c
c      dphi=abs(eta_to(1))*(dp(2)*cos(eta_to(2))-
c     +     dp(1)*sin(eta_to(2)))/klmn_a
c
c      dkappa=-abs(eta_to(1))*eta_to(1)*(dp(1)*cos(eta_to(2))+
c     +    dp(2)*sin(eta_to(2)))/klmn_a
c
c      dt=dp(3)*abs(eta_to(1))/klmn_a+eta_to(4)*dkappa/eta_to(1)
c
c     and finally update the track parameters
c
c      eta_to(1)=eta_to(1)-dkappa
c      eta_to(2)=eta_to(2)-dphi
c      eta_to(4)=eta_to(4)-dt
c
c     magnetic field correction is done

999   continue

      return

      end












