*
* $Id: stop_init.F,v 1.6 1996/06/27 05:36:55 clib Exp $
*
* $Log: stop_init.F,v $
* Revision 1.6  1996/06/27 05:36:55  clib
* Fixed x(i++1).
*
* Revision 1.5  1995/11/28 19:16:41  zfiles
* Bug fix Do not insert stops after hits with ttype=1
*
* Revision 1.4  1995/09/13  13:50:13  zfiles
* Mods from Rob. Bug fixes
*
* Revision 1.3  1995/09/05  13:16:34  zfiles
*  Use axial approximation to seed stereo-wire drift distance
* calculation.
*
* Revision 1.2  1995/08/29  13:29:00  zfiles
* Complete new set from Rob/Anders
*
*
c
C
C     Code created by 
C
C     Robert Kutschke : kutschke@lns598.lns.cornell.edu
C     Anders Ryd      : ryd@charm.physics.ucsb.edu
C
C     Created : Sept 1994 -- June 1995
C     Title   : stop_init
C     Purpose : This routine initializes the list of points in
C     the common block mater_stop where that fit will be stopped 
C     to add on hits or to add on material.
C     
C
C     Modifications:
C
C     Sept-4 95   Ryd: Added in a call to t_to_t_c for
C                      the stereo wires to avoid problem
C                      with finding a local minimum.
C     Sept-10 95  Ryd: Fixed the initialization of the
C                      stop-points to handle curlers and tracks
C                      that don't reach the DR correctly.
C     Oct-17 95   Ryd: Fixed bug so that stops not added after
C                      a ttype=1 hit.
C

      subroutine stop_init

      implicit none

      real eta0(5),deta(5),etap(5)
      real vetastar(5,5)

#include "klmn/klmnseq/cleofitcom.inc"
#include "klmn/klmnseq/klmncm.inc"
#include "klmn/klmnseq/klmntrk.inc"
#include "klmn/klmnseq/mater_stop.inc"
#include "klmn/klmnseq/klmn_misc.inc"
#include "klmn/klmnseq/klmnctrl.inc"

      integer i,j,fit_it,n_stop,i_stop,stop_n
      real    stop_r(50),q,dw,stereo_dist,psi
      real    v1xv2(3),ri,rim

      eta0(1)=k_start
      eta0(3)=d0_start
      eta0(2)=phi_start
      eta0(5)=z_start
      eta0(4)=t_start


      do 100,i=1,nhits


        used_in_fit(i)=.true.

        if (itype(i).eq.4) then
          y(i)=y_0(i)
          x(i)=x_0(i)
        elseif(itype(i).eq.1) then
          y(i)=y_0(i)
          x(i)=x_0(i)
        elseif(itype(i).eq.0) then
          y(i)=y_0(i)
          x(i)=x_0(i)
        elseif(itype(i).eq.3) then
          y(i)=y_0(i)
          x(i)=x_0(i)
        elseif(itype(i).eq.2) then


               call t_to_t_c(eta0,etap,
     +          xo_start-x_0(i),yo_start-y_0(i),zo_start,1)

               dw=stereo_dist(etap(1),etap(3),etap(2),
     +            etap(5),etap(4),
     +            0.0,0.0,cx(i),
     +            cy(i),psi,v1xv2)


                q=etap(1)/abs(etap(1))               

                x(i)=-etap(3)*sin(etap(2))+(sin(etap(2)+q*psi)-
     +                      sin(etap(2)))/(2*etap(1))+x_0(i)

                y(i)=etap(3)*cos(etap(2))-(cos(etap(2)+q*psi)-
     +                      cos(etap(2)))/(2*etap(1))+y_0(i)
         else
           print *,'Unknown type itype=',itype(i)          
         endif


 100  continue




C     build the stop points

c     should be different for svx and pt

      call vzero(next_type,max_point)
      call vzero(n_next_point,max_point)

      if (run_num.ge.70000) then
        stop_r(1)=0.0185
        stop_r(2)=0.023
        stop_r(3)=0.032
        stop_r(4)=0.0468
        stop_r(5)=0.07
        stop_r(6)=0.0781
        stop_r(7)=0.157
        stop_r(8)=0.170
        stop_r(9)=0.189
        n_stop=9
      else
        stop_r(1)=0.0345
        stop_r(2)=0.0369
        stop_r(3)=0.0447
        stop_r(4)=0.07
        stop_r(5)=0.0781
        stop_r(6)=0.157
        stop_r(7)=0.170
        stop_r(8)=0.189
        n_stop=8
      endif


      stop_n=0

      do 10,i=1,nhits

C       This is not a 100% perfect solution but it
C       solves the problem with having next_type(i).ne.0 for
C       hits with ttype=1, which is ilegal.

        if (ttype(i).eq.1) goto 10

        ri=sqrt(x(i)**2+y(i)**2)
        if (i.eq.nhits) then
          rim=0.0
        else
          rim=sqrt(x(i+1)**2+y(i+1)**2)
        endif

        if (ri.gt.rim) then

          do 15,i_stop=n_stop,1,-1

            if (stop_r(i_stop).gt.rim.and.
     +          stop_r(i_stop).lt.ri) then
              stop_n=stop_n+1
              x_stop_mat(stop_n)=stop_r(i_stop)
              z_stop_mat(stop_n)=0.0
              if (next_type(i).eq.0) next_type(i)=-stop_n
              n_next_point(i)=n_next_point(i)+1
            endif           

15        continue

        else

          do 25,i_stop=1,n_stop

            if (stop_r(i_stop).lt.rim.and.
     +          stop_r(i_stop).gt.ri) then
              stop_n=stop_n+1
              x_stop_mat(stop_n)=stop_r(i_stop)
              z_stop_mat(stop_n)=0.0
              if (next_type(i).eq.0) next_type(i)=-stop_n
              n_next_point(i)=n_next_point(i)+1
            endif           

25        continue

        endif

10    continue


      return

      end









