*
* $Id: tube_intercept.F,v 1.7 1996/04/23 23:55:32 zfiles Exp $
*
* $Log: tube_intercept.F,v $
* Revision 1.7  1996/04/23 23:55:32  zfiles
* Comments in previous version are wrong!
* True comment:
*      Bugfix for in/out through the z limit.
*
* Revision 1.6  1996/04/23 06:56:43  zfiles
* More mods from Rob.
*
* Revision 1.5  1996/04/08 14:51:29  zfiles
* Complete rewrite
*
* Revision 1.4  1996/03/12 20:06:38  zfiles
* Test for a precision problem with sqrt(ds)
*
* Revision 1.3  1995/09/05  13:16:55  zfiles
*  Remove diagnostic printout.
*
* Revision 1.2  1995/08/29  13:29:08  zfiles
* Complete new set from Rob/Anders
*
*
C 
C     Code created by 
C
C     Robert Kutschke : kutschke@lns598.lns.cornell.edu
C     Anders Ryd      : ryd@charm.physics.ucsb.edu
C
C     Created : Sept 1994 -- June 1995
C     Title   : tube_intercept
C     Purpose : This routine finds the points along a line from x1 to x2 where
C               the line enters or exits a tube with inner radius of rmin and
C               outer radius of rmax. The fractional distance at the entrance 
C               and exits points are stored in s_list and in_out_list flags if
C               the point was an entrance (+1) or an exit (-1). 
C               Cmp box_intercept. n_inter gives the number of entrance+exits.
C
C     
C
C     Modifications:
C     April 2   96  ryd:rewrote the routine to not have problems
C                   with square roots of negative numbers and
C                   also to treats the ends correctly.
C     April 19  96  ryd, comments added by kutschke.
C                   Bug fix for going in/out through z limits.
C


      subroutine tube_intercept(s_list,in_out_list,n_inter,
     +                             rmin,rmax,dz,x1,x2)


      implicit none

      real rmax,rmin,dz,x1(3),x2(3)
      real s_list(4)


      integer in_out_list(4),n_inter
 
      real*8 vxy2,vx1,smin,s1

      real*8 vxyinv,rmsq

      logical bubble
   
      integer i

      real*8 v(3),ds

   


      v(1)=dble(x2(1))-dble(x1(1))
      v(2)=dble(x2(2))-dble(x1(2))
      v(3)=dble(x2(3))-dble(x1(3))

      vxy2=v(1)**2+v(2)**2

      if (vxy2.eq.0.0) then
        n_inter=0.0
        goto 999
      endif

      vxyinv=1.0/sqrt(vxy2)

      vx1=dble(x1(1))*v(1)+dble(x1(2))*v(2)
      
      smin=-vx1/vxy2

      rmsq=(dble(x1(1))+v(1)*smin)**2+(dble(x1(2))+v(2)*smin)**2

      n_inter=0

      if (rmsq.lt.dble(rmax)**2) then
          ds=vxyinv*sqrt(dble(rmax)**2-rmsq)
          s1=smin-ds
          if (abs(s1-0.5).lt.0.5) then
            if (abs(dble(x1(3))+v(3)*s1).lt.dble(dz)) then
              n_inter=n_inter+1
              s_list(n_inter)=s1
              in_out_list(n_inter)=1
            endif
          endif
          s1=smin+ds
          if (abs(s1-0.5).lt.0.5) then
            if (abs(dble(x1(3))+v(3)*s1).lt.dble(dz)) then
              n_inter=n_inter+1
              s_list(n_inter)=s1
              in_out_list(n_inter)=-1
            endif
          endif
      endif

 

      if (rmsq.lt.dble(rmin)**2) then
          ds=vxyinv*sqrt(dble(rmin)**2-rmsq)
          s1=smin-ds
          if (abs(s1-0.5).lt.0.5) then
            if (abs(dble(x1(3))+v(3)*s1).lt.dble(dz)) then
              n_inter=n_inter+1
              s_list(n_inter)=s1
              in_out_list(n_inter)=-1
            endif
          endif
          s1=smin+ds
          if (abs(s1-0.5).lt.0.5) then
            if (abs(dble(x1(3))+v(3)*s1).lt.dble(dz)) then
              n_inter=n_inter+1
              s_list(n_inter)=s1
              in_out_list(n_inter)=1
            endif
          endif
      endif

  

      if (v(3).ne.0.0) then
        s1=(dble(dz)-dble(x1(3)))/v(3)
          if (abs(s1-0.5).lt.0.5) then
            rmsq=(dble(x1(1))+v(1)*s1)**2+(dble(x1(2))+v(2)*s1)**2
            if (rmsq.gt.dble(rmin)**2.and.rmsq.lt.dble(rmax)**2) then
              n_inter=n_inter+1 
              s_list(n_inter)=s1
              if (v(3).gt.0.0) then
                in_out_list(n_inter)=-1
              else
                in_out_list(n_inter)=+1
              endif
          endif      
        endif
        s1=(-dble(dz)-dble(x1(3)))/v(3)
          if (abs(s1-0.5).lt.0.5) then
            rmsq=(dble(x1(1))+v(1)*s1)**2+(dble(x1(2))+v(2)*s1)**2
            if (rmsq.gt.dble(rmin)**2.and.rmsq.lt.dble(rmax)**2) then
              n_inter=n_inter+1 
              s_list(n_inter)=s1
              if (v(3).gt.0.0) then
                in_out_list(n_inter)=+1
              else
                in_out_list(n_inter)=-1
              endif
          endif      
        endif
      endif

5     bubble=.false.

      do 10,i=1,n_inter-1
        if (s_list(i).gt.s_list(i+1)) then
          bubble=.true.
          call swapr(s_list(i),s_list(i+1))
          call swapr(in_out_list(i),in_out_list(i+1))
        endif
10    continue

      if (bubble) goto 5


999   continue

      return

      end





