*
* $Id: user_trans.F,v 1.5 1996/05/08 20:04:53 zfiles Exp $
*
* $Log: user_trans.F,v $
* Revision 1.5  1996/05/08 20:04:53  zfiles
* Comment out debug printout.
*
* Revision 1.4  1996/04/08 14:48:15  zfiles
* Return error flag if track stops in an outward swim
*
* Revision 1.3  1996/03/12 02:46:53  zfiles
* A bug fix from Anders Ryd
*
* Revision 1.2  1995/09/13  13:50:13  zfiles
* Mods from Rob. Bug fixes
*
* Revision 1.1  1995/09/05  13:18:16  zfiles
* New. Forgot to add before.
*
*
C
C     Code created by 
C
C     Robert Kutschke : kutschke@lns598.lns.cornell.edu
C     Anders Ryd      : ryd@charm.physics.ucsb.edu
C
C     Created : Aug 19 1995
C     Title   : user_trans
C     Purpose : To transport trackparameters. The effects of material
C               is incorporated here. Call the routine book_mater to
C               find how much material the track is passing through.
C               t_to_t transport parameters through vacuum. Intended
C               as a user routine to transport trackparameters, e.g.,
C               from the point of closest aproach to the z-axis to a 
C               vertex point.
C                  part_type    particle
C                      1           electron
C                      2           muon
C                      3           pion
C                      4           kaon
C                      5           proton
C     
C
C     Modifications:
C     Sept 12-95 Mods by ryd, comments by kutschke.
C                Decides itself whether to add or to subtract material.
C     Mars 11-96 ryd:Fixed bug in how the code decides if it is doing forward
C                or backward transport. Also modified the transposr to
C                be made with better precission. This is a cludge untill
C                the code is fixed so that it will not need small steps,
C                this will be outlined in the CBX on the Kalman filter
C                in the section about future improvements.
C     April 3-96 ryd:Added a new argument, ierr, to the routine. This
C                returns an error flag. IT is set to 0 if no error is
C                detected, and to 1. if the track stops, i.e., lost to much
C                energy.
C     
C
C
      subroutine user_trans(x_fr,y_fr,eta_fr,veta_fr,
     +           x_to,y_to,eta_to,veta_to,part_type,ierr)

      implicit none

      integer part_type,ierr
      real x_fr,y_fr,x_to,y_to,sc_fac,sc_fac1,sc_fac2,re,rk,rk2

      real eta_fr(5),eta_to(5),deta_fr(5),deta_to(5),ddk
      real*8  veta_fr(5,5),veta_to(5,5)
      real   eta_fr_s(5)

#include "klmn/klmnseq/cleofitcom.inc"
#include "klmn/klmnseq/mater_n.inc"
#include "klmn/klmnseq/klmnctrl.inc"
#include "klmn/klmnseq/klmn_par.inc"

      real eta_to_trans(5),deta_to_trans(5),e_fr,e_to
      real*8  veta_to_trans(5,5)
      real dk,v2phi,v2t,v2k,vkt,tsqrt,p,deltae
      real l,x_fr1,y_fr1,x_to1,y_to1,psi,forward
      integer n,ni
      integer j,k

      ierr=0

      l=sqrt((x_fr-x_to)**2+(y_fr-y_to)**2)

      n=l/0.01+1

      klmnctrl_part=part_type
      
      if (klmnctrl_part.eq.1) klmnctrl_mass=0.00511
      if (klmnctrl_part.eq.2) klmnctrl_mass=0.106
      if (klmnctrl_part.eq.3) klmnctrl_mass=0.13956
      if (klmnctrl_part.eq.4) klmnctrl_mass=0.497
      if (klmnctrl_part.eq.5) klmnctrl_mass=0.938


      klmnctrl_in=-1

      if (eta_fr(1).gt.0.0) then 
        klmnctrl_q=+1.0
      else
        klmnctrl_q=-1.0
      endif

C     Make the routine smart so that it knows if energyloss and
C     M.S. should be added or subtracted in the transport.
C
C     forward  = +1.0  if track transport is along trackmomentum
C                      i.e. energy loss and M.S. should be subtracted
C
C     forward  = -1.0  if track transport is oposite the trackmomentum
C                      i.e. energy loss and M.S. should be added



      call t_to_t(eta_fr,deta_fr,veta_fr,
     +         eta_to_trans,deta_to_trans,veta_to,
     +         x_fr-x_to,y_fr-y_to,0.0,0)

      psi=eta_fr(2)-eta_to_trans(2)

c      print *,'x_to,y_to:',x_to,y_to
c      print *,'eta_fr:',eta_fr(1),eta_fr(2),eta_fr(3)
c     +                 ,eta_fr(4),eta_fr(5)
c      print *,'q,psi:',klmnctrl_q,psi

      if (klmnctrl_q*psi.lt.0.0) then 
        forward=+1.0
      else
        forward=-1.0
      endif

c      if (forward.gt.0.0) print *,'Forward transport'
c      if (forward.lt.0.0) print *,'Backward transport'


      do 120,ni=1,n

        x_fr1=(x_to-x_fr)*(ni-1)/float(n)+x_fr
        y_fr1=(y_to-y_fr)*(ni-1)/float(n)+y_fr
 
        x_to1=(x_to-x_fr)*(ni)/float(n)+x_fr
        y_to1=(y_to-y_fr)*(ni)/float(n)+y_fr

        nmater_n=0

        klmnctrl_buildmater=.true.

        eta_fr_s(1)=eta_fr(1)
        eta_fr_s(2)=eta_fr(2)
        eta_fr_s(3)=eta_fr(3)
        eta_fr_s(4)=eta_fr(4)
        eta_fr_s(5)=eta_fr(5)

        call vzero(deta_fr,5)

        call t_to_t(eta_fr,deta_fr,veta_fr,
     +         eta_to_trans,deta_to_trans,veta_to,
     +         x_fr1-x_to1,y_fr1-y_to1,0.0,0)

         tsqrt=sqrt(1.0+eta_fr(4)**2)

         klmn_a=0.224844

         p=klmn_a*tsqrt/abs(eta_fr(1))

         e_fr=sqrt(klmnctrl_mass**2+p**2)


         eta_to(1)=eta_to_trans(1)
         eta_to(2)=eta_to_trans(2)
         eta_to(3)=eta_to_trans(3)
         eta_to(4)=eta_to_trans(4)
         eta_to(5)=eta_to_trans(5)

         call book_mater(eta_fr_s,x_fr1,y_fr1,0.0,eta_to,
     +           x_to1,y_to1,0.0,p,e_fr,deltae,
     +           v2phi,v2t,v2k,vkt,ddk)

         e_to=e_fr+klmnctrl_in*deltae*forward

C      put in a check to see if the track has **kinetic**
C      energy below 1MeV. If so set the kinetic energy to 1MeV
C      and set ierr=1. This flaggs that the track is considered 
C      to be a stopper.
  
         if (e_to.lt.klmnctrl_mass+0.001) then
           e_to=klmnctrl_mass+0.001
           ierr=1
         endif


         dk=+klmn_a*tsqrt*
     +          (1.0/sqrt(e_to**2-klmnctrl_mass**2)-1.0/p)

         if (eta_fr(1).gt.0.0) dk=-dk

         eta_to(1)=eta_to(1)+klmnctrl_in*dk*forward



         deta_to(1)=deta_to_trans(1)
         deta_to(2)=deta_to_trans(2)
         deta_to(3)=deta_to_trans(3)
         deta_to(4)=deta_to_trans(4)
         deta_to(5)=deta_to_trans(5)


         re=e_to/e_fr
         rk=eta_to(1)/(eta_to(1)-klmnctrl_in*dk*forward)
         rk2=rk*rk

         sc_fac1=re*rk*rk2
         sc_fac2=eta_to(4)*eta_to(1)*(1.0-re*rk2)/(1.0+eta_to(4)**2)



          veta_to(1,1)=veta_to(1,1)*sc_fac1**2+
     +               veta_to(1,4)*2*sc_fac1*sc_fac2+
     +               veta_to(4,4)*sc_fac2**2-forward*(v2k+ddk)
          veta_to(1,2)=veta_to(1,2)*sc_fac1+veta_to(4,2)*sc_fac2
          veta_to(2,1)=veta_to(1,2)
          veta_to(1,3)=veta_to(1,3)*sc_fac1+veta_to(4,3)*sc_fac2
          veta_to(3,1)=veta_to(1,3)
          veta_to(1,5)=veta_to(1,5)*sc_fac1+veta_to(4,5)*sc_fac2
          veta_to(5,1)=veta_to(1,5)
          veta_to(1,4)=veta_to(1,4)*sc_fac1+veta_to(4,4)*sc_fac2-
     +             forward*vkt
          veta_to(2,2)=veta_to(2,2)-forward*v2phi
          veta_to(4,1)=veta_to(1,4)
          veta_to(4,4)=veta_to(4,4)-forward*v2t


        call ucopy(eta_to,eta_fr,5)
        call ucopy(veta_to,veta_fr,50)

120   continue 



      return

      end










